# coding: utf-8

"""
    ORCID Member

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: Latest
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from pprint import pformat
from six import iteritems
import re


class Notification(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, source=None, put_code=None, notification_type=None, created_date=None, sent_date=None, read_date=None, archived_date=None):
        """
        Notification - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'source': 'Source',
            'put_code': 'int',
            'notification_type': 'str',
            'created_date': 'datetime',
            'sent_date': 'datetime',
            'read_date': 'datetime',
            'archived_date': 'datetime'
        }

        self.attribute_map = {
            'source': 'source',
            'put_code': 'put-code',
            'notification_type': 'notification-type',
            'created_date': 'created-date',
            'sent_date': 'sent-date',
            'read_date': 'read-date',
            'archived_date': 'archived-date'
        }

        self._source = source
        self._put_code = put_code
        self._notification_type = notification_type
        self._created_date = created_date
        self._sent_date = sent_date
        self._read_date = read_date
        self._archived_date = archived_date

    @property
    def source(self):
        """
        Gets the source of this Notification.

        :return: The source of this Notification.
        :rtype: Source
        """
        return self._source

    @source.setter
    def source(self, source):
        """
        Sets the source of this Notification.

        :param source: The source of this Notification.
        :type: Source
        """

        self._source = source

    @property
    def put_code(self):
        """
        Gets the put_code of this Notification.

        :return: The put_code of this Notification.
        :rtype: int
        """
        return self._put_code

    @put_code.setter
    def put_code(self, put_code):
        """
        Sets the put_code of this Notification.

        :param put_code: The put_code of this Notification.
        :type: int
        """

        self._put_code = put_code

    @property
    def notification_type(self):
        """
        Gets the notification_type of this Notification.

        :return: The notification_type of this Notification.
        :rtype: str
        """
        return self._notification_type

    @notification_type.setter
    def notification_type(self, notification_type):
        """
        Sets the notification_type of this Notification.

        :param notification_type: The notification_type of this Notification.
        :type: str
        """
        allowed_values = ["CUSTOM", "INSTITUTIONAL_CONNECTION", "PERMISSION", "AMENDED"]
        if notification_type not in allowed_values:
            raise ValueError(
                "Invalid value for `notification_type` ({0}), must be one of {1}"
                .format(notification_type, allowed_values)
            )

        self._notification_type = notification_type

    @property
    def created_date(self):
        """
        Gets the created_date of this Notification.

        :return: The created_date of this Notification.
        :rtype: datetime
        """
        return self._created_date

    @created_date.setter
    def created_date(self, created_date):
        """
        Sets the created_date of this Notification.

        :param created_date: The created_date of this Notification.
        :type: datetime
        """

        self._created_date = created_date

    @property
    def sent_date(self):
        """
        Gets the sent_date of this Notification.

        :return: The sent_date of this Notification.
        :rtype: datetime
        """
        return self._sent_date

    @sent_date.setter
    def sent_date(self, sent_date):
        """
        Sets the sent_date of this Notification.

        :param sent_date: The sent_date of this Notification.
        :type: datetime
        """

        self._sent_date = sent_date

    @property
    def read_date(self):
        """
        Gets the read_date of this Notification.

        :return: The read_date of this Notification.
        :rtype: datetime
        """
        return self._read_date

    @read_date.setter
    def read_date(self, read_date):
        """
        Sets the read_date of this Notification.

        :param read_date: The read_date of this Notification.
        :type: datetime
        """

        self._read_date = read_date

    @property
    def archived_date(self):
        """
        Gets the archived_date of this Notification.

        :return: The archived_date of this Notification.
        :rtype: datetime
        """
        return self._archived_date

    @archived_date.setter
    def archived_date(self, archived_date):
        """
        Sets the archived_date of this Notification.

        :param archived_date: The archived_date of this Notification.
        :type: datetime
        """

        self._archived_date = archived_date

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        if not isinstance(other, Notification):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other
